import datetime
import random
from http import HTTPStatus
from time import sleep
from typing import Any, Dict, List, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_error import HttpError
from ...models.http_validation_error import HTTPValidationError
from ...models.list_response_message_out import ListResponseMessageOut
from ...types import UNSET, Response, Unset

SLEEP_TIME = 0.05
NUM_RETRIES = 3


def _get_kwargs(
    app_id: str,
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, None, int] = UNSET,
    iterator: Union[Unset, None, str] = UNSET,
    channel: Union[Unset, None, str] = UNSET,
    before: Union[Unset, None, datetime.datetime] = UNSET,
    after: Union[Unset, None, datetime.datetime] = UNSET,
    with_content: Union[Unset, None, bool] = False,
    event_types: Union[Unset, None, List[str]] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/v1/app/{app_id}/msg/".format(client.base_url, app_id=app_id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["limit"] = limit

    params["iterator"] = iterator

    params["channel"] = channel

    json_before: Union[Unset, None, str] = UNSET
    if not isinstance(before, Unset):
        json_before = before.isoformat() if before else None

    params["before"] = json_before

    json_after: Union[Unset, None, str] = UNSET
    if not isinstance(after, Unset):
        json_after = after.isoformat() if after else None

    params["after"] = json_after

    params["with_content"] = with_content

    json_event_types: Union[Unset, None, List[str]] = UNSET
    if not isinstance(event_types, Unset):
        if event_types is None:
            json_event_types = None
        else:
            json_event_types = event_types

    params["event_types"] = json_event_types

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
        "params": params,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> ListResponseMessageOut:
    if response.status_code == HTTPStatus.OK:
        response_200 = ListResponseMessageOut.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.FORBIDDEN:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.NOT_FOUND:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.CONFLICT:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        raise HTTPValidationError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.TOO_MANY_REQUESTS:
        raise HttpError.init_exception(response.json(), response.status_code)
    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(*, client: Client, response: httpx.Response) -> Response[ListResponseMessageOut]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    app_id: str,
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, None, int] = UNSET,
    iterator: Union[Unset, None, str] = UNSET,
    channel: Union[Unset, None, str] = UNSET,
    before: Union[Unset, None, datetime.datetime] = UNSET,
    after: Union[Unset, None, datetime.datetime] = UNSET,
    with_content: Union[Unset, None, bool] = False,
    event_types: Union[Unset, None, List[str]] = UNSET,
) -> Response[ListResponseMessageOut]:
    """List Messages

     List all of the application's messages.

    The `before` and `after` parameters let you filter all items created before or after a certain date.
    These can be used alongside an iterator to paginate over results
    within a certain window.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        limit (Union[Unset, None, int]):
        iterator (Union[Unset, None, str]): The msg's ID Example: msg_1srOrx2ZWZBpBUvZwXKQmoEYga2.
        channel (Union[Unset, None, str]):  Example: project_1337.
        before (Union[Unset, None, datetime.datetime]):
        after (Union[Unset, None, datetime.datetime]):
        with_content (Union[Unset, None, bool]):
        event_types (Union[Unset, None, List[str]]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ListResponseMessageOut]
    """

    kwargs = _get_kwargs(
        app_id=app_id,
        client=client,
        limit=limit,
        iterator=iterator,
        channel=channel,
        before=before,
        after=after,
        with_content=with_content,
        event_types=event_types,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    for retry_count in range(NUM_RETRIES):
        response = httpx.request(
            verify=client.verify_ssl,
            **kwargs,
        )
        if response.status_code >= 500:
            kwargs["headers"]["svix-retry-count"] = str(retry_count)
            sleep(SLEEP_TIME * (2**retry_count))
        else:
            break

    return _build_response(client=client, response=response)


def sync(
    app_id: str,
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, None, int] = UNSET,
    iterator: Union[Unset, None, str] = UNSET,
    channel: Union[Unset, None, str] = UNSET,
    before: Union[Unset, None, datetime.datetime] = UNSET,
    after: Union[Unset, None, datetime.datetime] = UNSET,
    with_content: Union[Unset, None, bool] = False,
    event_types: Union[Unset, None, List[str]] = UNSET,
) -> ListResponseMessageOut:
    """List Messages

     List all of the application's messages.

    The `before` and `after` parameters let you filter all items created before or after a certain date.
    These can be used alongside an iterator to paginate over results
    within a certain window.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        limit (Union[Unset, None, int]):
        iterator (Union[Unset, None, str]): The msg's ID Example: msg_1srOrx2ZWZBpBUvZwXKQmoEYga2.
        channel (Union[Unset, None, str]):  Example: project_1337.
        before (Union[Unset, None, datetime.datetime]):
        after (Union[Unset, None, datetime.datetime]):
        with_content (Union[Unset, None, bool]):
        event_types (Union[Unset, None, List[str]]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ListResponseMessageOut
    """

    return sync_detailed(
        app_id=app_id,
        client=client,
        limit=limit,
        iterator=iterator,
        channel=channel,
        before=before,
        after=after,
        with_content=with_content,
        event_types=event_types,
    ).parsed


async def asyncio_detailed(
    app_id: str,
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, None, int] = UNSET,
    iterator: Union[Unset, None, str] = UNSET,
    channel: Union[Unset, None, str] = UNSET,
    before: Union[Unset, None, datetime.datetime] = UNSET,
    after: Union[Unset, None, datetime.datetime] = UNSET,
    with_content: Union[Unset, None, bool] = False,
    event_types: Union[Unset, None, List[str]] = UNSET,
) -> Response[ListResponseMessageOut]:
    """List Messages

     List all of the application's messages.

    The `before` and `after` parameters let you filter all items created before or after a certain date.
    These can be used alongside an iterator to paginate over results
    within a certain window.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        limit (Union[Unset, None, int]):
        iterator (Union[Unset, None, str]): The msg's ID Example: msg_1srOrx2ZWZBpBUvZwXKQmoEYga2.
        channel (Union[Unset, None, str]):  Example: project_1337.
        before (Union[Unset, None, datetime.datetime]):
        after (Union[Unset, None, datetime.datetime]):
        with_content (Union[Unset, None, bool]):
        event_types (Union[Unset, None, List[str]]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ListResponseMessageOut]
    """

    kwargs = _get_kwargs(
        app_id=app_id,
        client=client,
        limit=limit,
        iterator=iterator,
        channel=channel,
        before=before,
        after=after,
        with_content=with_content,
        event_types=event_types,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        for retry_count in range(NUM_RETRIES):
            response = await _client.request(**kwargs)
            if response.status_code >= 500:
                kwargs["headers"]["svix-retry-count"] = str(retry_count)
                sleep(SLEEP_TIME * (2**retry_count))
            else:
                break

    return _build_response(client=client, response=response)


async def asyncio(
    app_id: str,
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, None, int] = UNSET,
    iterator: Union[Unset, None, str] = UNSET,
    channel: Union[Unset, None, str] = UNSET,
    before: Union[Unset, None, datetime.datetime] = UNSET,
    after: Union[Unset, None, datetime.datetime] = UNSET,
    with_content: Union[Unset, None, bool] = False,
    event_types: Union[Unset, None, List[str]] = UNSET,
) -> ListResponseMessageOut:
    """List Messages

     List all of the application's messages.

    The `before` and `after` parameters let you filter all items created before or after a certain date.
    These can be used alongside an iterator to paginate over results
    within a certain window.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        limit (Union[Unset, None, int]):
        iterator (Union[Unset, None, str]): The msg's ID Example: msg_1srOrx2ZWZBpBUvZwXKQmoEYga2.
        channel (Union[Unset, None, str]):  Example: project_1337.
        before (Union[Unset, None, datetime.datetime]):
        after (Union[Unset, None, datetime.datetime]):
        with_content (Union[Unset, None, bool]):
        event_types (Union[Unset, None, List[str]]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ListResponseMessageOut
    """

    return (
        await asyncio_detailed(
            app_id=app_id,
            client=client,
            limit=limit,
            iterator=iterator,
            channel=channel,
            before=before,
            after=after,
            with_content=with_content,
            event_types=event_types,
        )
    ).parsed



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.event_type_out_schemas import EventTypeOutSchemas


T = TypeVar("T", bound="EventTypeOut")


@attr.s(auto_attribs=True)
class EventTypeOut:
    """
    Attributes:
        created_at (datetime.datetime):
        description (str):  Example: A user has signed up.
        name (str): The event type's name Example: user.signup.
        updated_at (datetime.datetime):
        archived (Union[Unset, bool]):
        feature_flag (Union[Unset, None, str]):  Example: cool-new-feature.
        schemas (Union[Unset, None, EventTypeOutSchemas]): The schema for the event type for a specific version as a
            JSON schema. Example: {'1': {'description': 'An invoice was paid by a user', 'properties': {'invoiceId':
            {'description': 'The invoice id', 'type': 'string'}, 'userId': {'description': 'The user id', 'type':
            'string'}}, 'required': ['invoiceId', 'userId'], 'title': 'Invoice Paid Event', 'type': 'object'}}.
    """

    created_at: datetime.datetime
    description: str
    name: str
    updated_at: datetime.datetime
    archived: Union[Unset, bool] = False
    feature_flag: Union[Unset, None, str] = UNSET
    schemas: Union[Unset, None, "EventTypeOutSchemas"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        created_at = self.created_at.isoformat()

        description = self.description
        name = self.name
        updated_at = self.updated_at.isoformat()

        archived = self.archived
        feature_flag = self.feature_flag
        schemas = self.schemas

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "createdAt": created_at,
                "description": description,
                "name": name,
                "updatedAt": updated_at,
            }
        )
        if archived is not UNSET:
            field_dict["archived"] = archived
        if feature_flag is not UNSET:
            field_dict["featureFlag"] = feature_flag
        if schemas is not UNSET:
            field_dict["schemas"] = schemas

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        created_at = isoparse(d.pop("createdAt"))

        description = d.pop("description")

        name = d.pop("name")

        updated_at = isoparse(d.pop("updatedAt"))

        archived = d.pop("archived", UNSET)

        feature_flag = d.pop("featureFlag", UNSET)

        schemas = d.pop("schemas", UNSET)

        event_type_out = cls(
            created_at=created_at,
            description=description,
            name=name,
            updated_at=updated_at,
            archived=archived,
            feature_flag=feature_flag,
            schemas=schemas,
        )

        event_type_out.additional_properties = d
        return event_type_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .app_portal_access_in import AppPortalAccessIn
from .app_portal_access_out import AppPortalAccessOut
from .app_usage_stats_in import AppUsageStatsIn
from .app_usage_stats_out import AppUsageStatsOut
from .application_in import ApplicationIn
from .application_in_metadata import ApplicationInMetadata
from .application_out import ApplicationOut
from .application_out_metadata import ApplicationOutMetadata
from .application_patch import ApplicationPatch
from .application_patch_metadata import ApplicationPatchMetadata
from .application_stats import ApplicationStats
from .application_token_expire_in import ApplicationTokenExpireIn
from .attempt_statistics_data import AttemptStatisticsData
from .attempt_statistics_response import AttemptStatisticsResponse
from .background_task_data import BackgroundTaskData
from .background_task_out import BackgroundTaskOut
from .background_task_status import BackgroundTaskStatus
from .background_task_type import BackgroundTaskType
from .border_radius_config import BorderRadiusConfig
from .border_radius_enum import BorderRadiusEnum
from .custom_color_palette import CustomColorPalette
from .custom_theme_override import CustomThemeOverride
from .dashboard_access_out import DashboardAccessOut
from .endpoint_created_event import EndpointCreatedEvent
from .endpoint_created_event_data import EndpointCreatedEventData
from .endpoint_created_event_type import EndpointCreatedEventType
from .endpoint_deleted_event import EndpointDeletedEvent
from .endpoint_deleted_event_data import EndpointDeletedEventData
from .endpoint_deleted_event_type import EndpointDeletedEventType
from .endpoint_disabled_event import EndpointDisabledEvent
from .endpoint_disabled_event_data import EndpointDisabledEventData
from .endpoint_disabled_event_type import EndpointDisabledEventType
from .endpoint_headers_in import EndpointHeadersIn
from .endpoint_headers_in_headers import EndpointHeadersInHeaders
from .endpoint_headers_out import EndpointHeadersOut
from .endpoint_headers_out_headers import EndpointHeadersOutHeaders
from .endpoint_headers_patch_in import EndpointHeadersPatchIn
from .endpoint_headers_patch_in_headers import EndpointHeadersPatchInHeaders
from .endpoint_in import EndpointIn
from .endpoint_in_metadata import EndpointInMetadata
from .endpoint_message_out import EndpointMessageOut
from .endpoint_message_out_payload import EndpointMessageOutPayload
from .endpoint_out import EndpointOut
from .endpoint_out_metadata import EndpointOutMetadata
from .endpoint_patch import EndpointPatch
from .endpoint_patch_metadata import EndpointPatchMetadata
from .endpoint_secret_out import EndpointSecretOut
from .endpoint_secret_rotate_in import EndpointSecretRotateIn
from .endpoint_stats import EndpointStats
from .endpoint_transformation_in import EndpointTransformationIn
from .endpoint_transformation_out import EndpointTransformationOut
from .endpoint_transformation_simulate_in import EndpointTransformationSimulateIn
from .endpoint_transformation_simulate_in_payload import EndpointTransformationSimulateInPayload
from .endpoint_transformation_simulate_out import EndpointTransformationSimulateOut
from .endpoint_update import EndpointUpdate
from .endpoint_update_metadata import EndpointUpdateMetadata
from .endpoint_updated_event import EndpointUpdatedEvent
from .endpoint_updated_event_data import EndpointUpdatedEventData
from .endpoint_updated_event_type import EndpointUpdatedEventType
from .environment_in import EnvironmentIn
from .environment_out import EnvironmentOut
from .environment_settings_out import EnvironmentSettingsOut
from .event_example_in import EventExampleIn
from .event_type_example_out import EventTypeExampleOut
from .event_type_example_out_example import EventTypeExampleOutExample
from .event_type_example_out_example_additional_property import EventTypeExampleOutExampleAdditionalProperty
from .event_type_in import EventTypeIn
from .event_type_in_schemas import EventTypeInSchemas
from .event_type_in_schemas_additional_property import EventTypeInSchemasAdditionalProperty
from .event_type_out import EventTypeOut
from .event_type_out_schemas import EventTypeOutSchemas
from .event_type_out_schemas_additional_property import EventTypeOutSchemasAdditionalProperty
from .event_type_patch import EventTypePatch
from .event_type_patch_schemas import EventTypePatchSchemas
from .event_type_patch_schemas_additional_property import EventTypePatchSchemasAdditionalProperty
from .event_type_schema_in import EventTypeSchemaIn
from .event_type_schema_in_schema import EventTypeSchemaInSchema
from .event_type_update import EventTypeUpdate
from .event_type_update_schemas import EventTypeUpdateSchemas
from .event_type_update_schemas_additional_property import EventTypeUpdateSchemasAdditionalProperty
from .export_environment_in import ExportEnvironmentIn
from .font_size_config import FontSizeConfig
from .http_error import HttpError
from .http_validation_error import HTTPValidationError
from .integration_in import IntegrationIn
from .integration_key_out import IntegrationKeyOut
from .integration_out import IntegrationOut
from .integration_update import IntegrationUpdate
from .list_response_application_out import ListResponseApplicationOut
from .list_response_application_stats import ListResponseApplicationStats
from .list_response_background_task_out import ListResponseBackgroundTaskOut
from .list_response_endpoint_message_out import ListResponseEndpointMessageOut
from .list_response_endpoint_out import ListResponseEndpointOut
from .list_response_event_type_out import ListResponseEventTypeOut
from .list_response_integration_out import ListResponseIntegrationOut
from .list_response_message_attempt_endpoint_out import ListResponseMessageAttemptEndpointOut
from .list_response_message_attempt_out import ListResponseMessageAttemptOut
from .list_response_message_endpoint_out import ListResponseMessageEndpointOut
from .list_response_message_out import ListResponseMessageOut
from .message_attempt_endpoint_out import MessageAttemptEndpointOut
from .message_attempt_exhausted_event import MessageAttemptExhaustedEvent
from .message_attempt_exhausted_event_data import MessageAttemptExhaustedEventData
from .message_attempt_exhausted_event_type import MessageAttemptExhaustedEventType
from .message_attempt_failed_data import MessageAttemptFailedData
from .message_attempt_failing_event import MessageAttemptFailingEvent
from .message_attempt_failing_event_data import MessageAttemptFailingEventData
from .message_attempt_failing_event_type import MessageAttemptFailingEventType
from .message_attempt_headers_out import MessageAttemptHeadersOut
from .message_attempt_headers_out_sent_headers import MessageAttemptHeadersOutSentHeaders
from .message_attempt_out import MessageAttemptOut
from .message_attempt_trigger_type import MessageAttemptTriggerType
from .message_broadcast_in import MessageBroadcastIn
from .message_broadcast_in_payload import MessageBroadcastInPayload
from .message_broadcast_out import MessageBroadcastOut
from .message_endpoint_out import MessageEndpointOut
from .message_in import MessageIn
from .message_in_payload import MessageInPayload
from .message_out import MessageOut
from .message_out_payload import MessageOutPayload
from .message_status import MessageStatus
from .one_time_token_in import OneTimeTokenIn
from .one_time_token_out import OneTimeTokenOut
from .ordering import Ordering
from .recover_in import RecoverIn
from .recover_out import RecoverOut
from .replay_in import ReplayIn
from .replay_out import ReplayOut
from .settings_in import SettingsIn
from .settings_out import SettingsOut
from .statistics_period import StatisticsPeriod
from .status_code_class import StatusCodeClass
from .transformation_http_method import TransformationHttpMethod
from .validation_error import ValidationError

__all__ = (
    "ApplicationIn",
    "ApplicationInMetadata",
    "ApplicationOut",
    "ApplicationOutMetadata",
    "ApplicationPatch",
    "ApplicationPatchMetadata",
    "ApplicationStats",
    "ApplicationTokenExpireIn",
    "AppPortalAccessIn",
    "AppPortalAccessOut",
    "AppUsageStatsIn",
    "AppUsageStatsOut",
    "AttemptStatisticsData",
    "AttemptStatisticsResponse",
    "BackgroundTaskData",
    "BackgroundTaskOut",
    "BackgroundTaskStatus",
    "BackgroundTaskType",
    "BorderRadiusConfig",
    "BorderRadiusEnum",
    "CustomColorPalette",
    "CustomThemeOverride",
    "DashboardAccessOut",
    "EndpointCreatedEvent",
    "EndpointCreatedEventData",
    "EndpointCreatedEventType",
    "EndpointDeletedEvent",
    "EndpointDeletedEventData",
    "EndpointDeletedEventType",
    "EndpointDisabledEvent",
    "EndpointDisabledEventData",
    "EndpointDisabledEventType",
    "EndpointHeadersIn",
    "EndpointHeadersInHeaders",
    "EndpointHeadersOut",
    "EndpointHeadersOutHeaders",
    "EndpointHeadersPatchIn",
    "EndpointHeadersPatchInHeaders",
    "EndpointIn",
    "EndpointInMetadata",
    "EndpointMessageOut",
    "EndpointMessageOutPayload",
    "EndpointOut",
    "EndpointOutMetadata",
    "EndpointPatch",
    "EndpointPatchMetadata",
    "EndpointSecretOut",
    "EndpointSecretRotateIn",
    "EndpointStats",
    "EndpointTransformationIn",
    "EndpointTransformationOut",
    "EndpointTransformationSimulateIn",
    "EndpointTransformationSimulateInPayload",
    "EndpointTransformationSimulateOut",
    "EndpointUpdate",
    "EndpointUpdatedEvent",
    "EndpointUpdatedEventData",
    "EndpointUpdatedEventType",
    "EndpointUpdateMetadata",
    "EnvironmentIn",
    "EnvironmentOut",
    "EnvironmentSettingsOut",
    "EventExampleIn",
    "EventTypeExampleOut",
    "EventTypeExampleOutExample",
    "EventTypeExampleOutExampleAdditionalProperty",
    "EventTypeIn",
    "EventTypeInSchemas",
    "EventTypeInSchemasAdditionalProperty",
    "EventTypeOut",
    "EventTypeOutSchemas",
    "EventTypeOutSchemasAdditionalProperty",
    "EventTypePatch",
    "EventTypePatchSchemas",
    "EventTypePatchSchemasAdditionalProperty",
    "EventTypeSchemaIn",
    "EventTypeSchemaInSchema",
    "EventTypeUpdate",
    "EventTypeUpdateSchemas",
    "EventTypeUpdateSchemasAdditionalProperty",
    "ExportEnvironmentIn",
    "FontSizeConfig",
    "HttpError",
    "HTTPValidationError",
    "IntegrationIn",
    "IntegrationKeyOut",
    "IntegrationOut",
    "IntegrationUpdate",
    "ListResponseApplicationOut",
    "ListResponseApplicationStats",
    "ListResponseBackgroundTaskOut",
    "ListResponseEndpointMessageOut",
    "ListResponseEndpointOut",
    "ListResponseEventTypeOut",
    "ListResponseIntegrationOut",
    "ListResponseMessageAttemptEndpointOut",
    "ListResponseMessageAttemptOut",
    "ListResponseMessageEndpointOut",
    "ListResponseMessageOut",
    "MessageAttemptEndpointOut",
    "MessageAttemptExhaustedEvent",
    "MessageAttemptExhaustedEventData",
    "MessageAttemptExhaustedEventType",
    "MessageAttemptFailedData",
    "MessageAttemptFailingEvent",
    "MessageAttemptFailingEventData",
    "MessageAttemptFailingEventType",
    "MessageAttemptHeadersOut",
    "MessageAttemptHeadersOutSentHeaders",
    "MessageAttemptOut",
    "MessageAttemptTriggerType",
    "MessageBroadcastIn",
    "MessageBroadcastInPayload",
    "MessageBroadcastOut",
    "MessageEndpointOut",
    "MessageIn",
    "MessageInPayload",
    "MessageOut",
    "MessageOutPayload",
    "MessageStatus",
    "OneTimeTokenIn",
    "OneTimeTokenOut",
    "Ordering",
    "RecoverIn",
    "RecoverOut",
    "ReplayIn",
    "ReplayOut",
    "SettingsIn",
    "SettingsOut",
    "StatisticsPeriod",
    "StatusCodeClass",
    "TransformationHttpMethod",
    "ValidationError",
)



from .internal.openapi_client.models.http_error import HttpError
from .internal.openapi_client.models.http_validation_error import HTTPValidationError
from .webhooks import WebhookVerificationError

__all__ = [
    "HTTPValidationError",
    "HttpError",
    "WebhookVerificationError",
]



from .api import (
    ApplicationIn,
    ApplicationOut,
    DashboardAccessOut,
    IntegrationIn,
    IntegrationKeyOut,
    IntegrationOut,
    IntegrationUpdate,
    ListResponseApplicationOut,
    ListResponseIntegrationOut,
    ListResponseMessageOut,
    MessageIn,
    MessageOut,
    Svix,
    SvixAsync,
    SvixOptions,
)
from .webhooks import Webhook, WebhookVerificationError

__all__ = [
    "ApplicationIn",
    "ApplicationOut",
    "DashboardAccessOut",
    "ListResponseApplicationOut",
    "ListResponseIntegrationOut",
    "ListResponseMessageOut",
    "IntegrationIn",
    "IntegrationKeyOut",
    "IntegrationOut",
    "IntegrationUpdate",
    "MessageIn",
    "MessageOut",
    "Svix",
    "SvixAsync",
    "SvixOptions",
    "Webhook",
    "WebhookVerificationError",
]

__version__ = "1.5.0"



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.message_attempt_out import MessageAttemptOut


T = TypeVar("T", bound="ListResponseMessageAttemptOut")


@attr.s(auto_attribs=True)
class ListResponseMessageAttemptOut:
    """
    Attributes:
        data (List['MessageAttemptOut']):
        done (bool):
        iterator (Union[Unset, None, str]):  Example: iterator.
        prev_iterator (Union[Unset, None, str]):  Example: -iterator.
    """

    data: List["MessageAttemptOut"]
    done: bool
    iterator: Union[Unset, None, str] = UNSET
    prev_iterator: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        data = []
        for data_item_data in self.data:
            data_item = data_item_data.to_dict()

            data.append(data_item)

        done = self.done
        iterator = self.iterator
        prev_iterator = self.prev_iterator

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "data": data,
                "done": done,
            }
        )
        if iterator is not UNSET:
            field_dict["iterator"] = iterator
        if prev_iterator is not UNSET:
            field_dict["prevIterator"] = prev_iterator

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.message_attempt_out import MessageAttemptOut

        d = src_dict.copy()
        data = []
        _data = d.pop("data")
        for data_item_data in _data:
            data_item = MessageAttemptOut.from_dict(data_item_data)

            data.append(data_item)

        done = d.pop("done")

        iterator = d.pop("iterator", UNSET)

        prev_iterator = d.pop("prevIterator", UNSET)

        list_response_message_attempt_out = cls(
            data=data,
            done=done,
            iterator=iterator,
            prev_iterator=prev_iterator,
        )

        list_response_message_attempt_out.additional_properties = d
        return list_response_message_attempt_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
